unit uLog;

interface

{  27 january 2001
      Roberto de Martin Serqueira (martinsc@uol.com.br) found some of weakness
      in DebugStr when it had to handle very large strings. When displaying
      very large strings the routine wat too slow. This is now improved in
      speed and stability }

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Menus;

type
  TLog = class(TForm)
    LogWnd: TRichEdit;
    PopupMenu: TPopupMenu;
    Copy1: TMenuItem;
    Clear1: TMenuItem;
    Refresh1: TMenuItem;
    LineMode1: TMenuItem;
    Debugstring1: TMenuItem;
    Wordwrap1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure Copy1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure Refresh1Click(Sender: TObject);
    procedure LineMode1Click(Sender: TObject);
    procedure Debugstring1Click(Sender: TObject);
    procedure Wordwrap1Click(Sender: TObject);
  public
    TextColor: TColor;
    Header:    string;
    procedure Add(const ToLog: string);
  end;

var
  Log: TLog;

implementation

uses uMain;

{$R *.DFM}

//------------------------------------------------------------------------------
function _BinHex2Str(src: char; mul: byte; add: char): char;
asm
   cmp al, '9'
   jbe @Next
   sub al, 7
@Next:
   sub al, 30h
   mul dl
   add al, cl
end;

//------------------------------------------------------------------------------
function _Str2BinHex(src: byte): char;
asm
   cmp al, 9
   jbe @Next
   add al, 7
@Next:
   add al, 30h
end;

//------------------------------------------------------------------------------
function _DebugStrKoppel(n: byte): char;
asm
   cmp al, 7
   jne @Space
   mov al, '-'
   jmp @Ok
@Space:
   mov al, ' '
@Ok:
end;

//------------------------------------------------------------------------------
function _DebugStrLowChar(c: char): char;
asm
   cmp al, ' '
   jae @Ok
   mov al, '.'
@Ok:
end;

//------------------------------------------------------------------------------
function DebugStrLine(s, d: PChar; offset: word; Len: integer): integer;
var
   i, j, k: integer;
begin
   i := 0;
   j := 5;
   k := 53;
   inc(s, offset);
   move(IntToHex(offset, 4)[1], d[0], 4);
   d[4] := ' ';

   if len > 16 then len := 16;

   while i < len do
   begin
      d[j] := _Str2BinHex(Ord(s[i]) div 16);
      inc(j);
      d[j] := _Str2BinHex(Ord(s[i]) mod 16);
      inc(j);
      d[j] := _DebugStrKoppel(i);
      inc(j);
      d[k] := _DebugStrLowChar(s[i]);
      inc(k);
      inc(i);
   end;

   while j < 53 do
   begin
      d[j] := ' ';
      inc(j);
   end;

   d[k] := #0;
   Result := i;
end;

//------------------------------------------------------------------------------
function DebugStr(const Data: string; var Debug: TStrings): TStrings;
var
   s, d: PChar;
   i, len: integer;
begin
   len := Length(Data);
   i := 0;
   s := Pointer(Data);
   GetMem(d, 70);       // len of each string is 69 + terminating NULL

   while i < Len do
   begin
      inc(i, DebugStrLine(s, d, i, Len - i));
      Debug.Add(StrPas(d));
   end;

   Result := Debug;
   FreeMem(d);
end;

//------------------------------------------------------------------------------
function RemoveLowChar(const Source: string): string;
var n: integer;
begin
   Setlength(Result, length(Source));
   for n := 1 to length(Source) do
      Result[n] := _DebugStrLowChar(Source[n]);
end;

//------------------------------------------------------------------------------
procedure TLog.FormCreate(Sender: TObject);
begin
   LogWnd.Clear;
   LogWnd.Align := alClient;
   Header := '';
   TextColor := clBlack;
end;

//------------------------------------------------------------------------------
procedure TLog.Add(const ToLog: string);
var
   OrgPos: integer;
   Strings: TStrings;
begin
   if not visible then
      show;

   { Start adding lines in right color, but dont scroll to the end
     whilst adding lines, even if it has focus.
     Only if Refresh is checked then scroll to end }

   with LogWnd do
   begin
      OrgPos := SelStart;
      Lines.BeginUpdate;

      Strings := TStringList.Create;
      try
         SelStart := GetTextLen;
         SelAttributes.Color := TextColor;

         if (Lines.Count > 0) and (Lines[Lines.Count - 1] <> '') then
            Lines.Add('');
         Lines.Add(DateTimeToStr(Now) + ' ' + Header);

         if not DebugString1.Checked then
            Lines.Add(RemoveLowChar(ToLog))
         else
            Lines.AddStrings(DebugStr(ToLog, Strings));

         SelStart := OrgPos;
         Perform(EM_SCROLLCARET, 0, 0);  // not needed in Delphi 5
      finally
         Lines.EndUpdate;
         Strings.Free;
      end;

      if Refresh1.Checked then
      begin
         SelStart := GetTextLen;
         Perform(EM_SCROLLCARET, 0, 0);  // not needed in Delphi 5
      end;
   end;
end;

//------------------------------------------------------------------------------
procedure TLog.Copy1Click(Sender: TObject);
begin
   with LogWnd do
   begin
      SelectAll;
      CopyToClipboard;
      SelLength := 0;
   end;
end;

//------------------------------------------------------------------------------
procedure TLog.Clear1Click(Sender: TObject);
begin
   LogWnd.Clear;
end;

//------------------------------------------------------------------------------
procedure TLog.Refresh1Click(Sender: TObject);
begin
   with Sender as TMenuItem do
      Checked := not Checked;
end;

//------------------------------------------------------------------------------
procedure TLog.LineMode1Click(Sender: TObject);
var n: integer;
begin
   with Sender as TMenuItem do
   begin
      Checked := not Checked;
      with Main.WSocketServer do
         for n := 0 to ClientCount - 1 do
            with Client[n] as TClient do
            begin
               LineMode := Checked;
               RemoteSocket.LineMode := Checked;
            end;
   end;
end;

//------------------------------------------------------------------------------
procedure TLog.Debugstring1Click(Sender: TObject);
begin
   with Sender as TMenuItem do
      Checked := not Checked;
end;

//------------------------------------------------------------------------------
procedure TLog.Wordwrap1Click(Sender: TObject);
begin
   with Sender as TMenuItem do
   begin
      Checked := not Checked;
      LogWnd.WordWrap := Checked;
   end;
end;

end.
