unit uMain;

{  27 january 2001
      Changed CloseDelayed to ShutDown(1) when a remote close. In some
      circumstances there could be data not received by local socket when closed }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, WSocket, WSocketS, ComCtrls, ExtCtrls;

type
  TClient = class(TWSocketClient)
  private
    Rcvd: string;
    Color: TColor;
  public
    RemoteSocket: TWSocket;
    destructor destroy; override;
  end;

type
  TMain = class(TForm)
    WSocketServer: TWSocketServer;
    RemoteAddr: TEdit;
    RemotePort: TEdit;
    LocalPoort: TEdit;
    ListenBtn: TButton;
    StatusBar: TStatusBar;
    Bevel1: TBevel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    procedure ListenBtnClick(Sender: TObject);
    procedure WSocketServerClientConnect(Sender: TObject; Client: TWSocketClient; Error: Word);
    procedure WSocketServerClientDisconnect(Sender: TObject; Client: TWSocketClient; Error: Word);
    procedure BgException(Sender: TObject; E: Exception; var CanClose: Boolean);
    procedure WSocketServerSessionClosed(Sender: TObject; Error: Word);
  private
    procedure ClientDataAvailable(Sender: TObject; Error: Word);
    procedure RemoteSessionConnected(Sender: TObject; Error: Word);
    procedure RemoteDataAvailable(Sender: TObject; Error: Word);
    procedure RemoteSessionClosed(Sender: TObject; Error: Word);
    procedure RemoteDnsLookupDone(Sender: TObject; Error: Word);
  public
  end;

var
  Main: TMain;

implementation

uses uLog;

{$R *.DFM}

//------------------------------------------------------------------------------
destructor TClient.destroy;
begin
   if not application.terminated then  // Log is destroyed before Main
   begin
      Log.TextColor := Color;
      Log.Header    := 'Connection Closed';
      Log.Add('');
   end;
   RemoteSocket.Free;
   inherited;
end;

//------------------------------------------------------------------------------
procedure TMain.ListenBtnClick(Sender: TObject);
var n: integer;
begin
   if ListenBtn.Tag = 0 then
   begin
      with WSocketServer do
      begin
         Banner := '';
         BannerToBusy := '';
         ClientClass := TClient;
         Addr := '0.0.0.0';
         Port := LocalPoort.Text;
         Listen;
      end;
      LocalPoort.Enabled := False;
      RemotePort.Enabled := False;
      RemoteAddr.Enabled := False;
      ListenBtn.Caption  := 'Cancel';
      ListenBtn.Tag      :=  1;
   end else
   begin
      for n := 0 to WSocketServer.ClientCount - 1 do WSocketServer.Client[n].Close;
      WSocketServer.Close;
   end;
end;

//------------------------------------------------------------------------------
procedure TMain.WSocketServerClientConnect(Sender: TObject; Client: TWSocketClient; Error: Word);
const aColor: array[1..11] of TColor = (clBlack, clMaroon, clGreen, clOlive, clNavy, clPurple, clRed, clLime, clYellow, clBlue, clFuchsia);
begin
   with Client as TClient do
   begin
      Rcvd := '';
      OnDataAvailable        := ClientDataAvailable;
      OnBgException          := BgException;
      LineMode               := Log.LineMode1.Checked;
      LineEnd                := #13#10;
      RemoteSocket           := TWSocket.Create(Client); // needs a owner for his events to recognize TClient
      RemoteSocket.Port      := RemotePort.Text;
      RemoteSocket.LineMode           := Log.LineMode1.Checked;
      RemoteSocket.LineEnd            := #13#10;
      RemoteSocket.OnSessionConnected := RemoteSessionConnected;
      RemoteSocket.OnDataAvailable    := RemoteDataAvailable;
      RemoteSocket.OnSessionClosed    := RemoteSessionClosed;
      RemoteSocket.OnBgException      := BgException;
      RemoteSocket.OnDnsLookupDone    := RemoteDnsLookupDone;
      RemoteSocket.DnsLookup(RemoteAddr.Text);
      StatusBar.Panels[1].Text := IntToStr(TWSocketServer(Server).ClientCount);
      StatusBar.Panels[3].Text := GetPeerAddr;
      if TWSocketServer(Server).ClientCount <= high(aColor) then Color := aColor[TWSocketServer(Server).ClientCount]
      else Color := clBlack;
   end;
end;

//------------------------------------------------------------------------------
procedure TMain.RemoteDnsLookupDone(Sender: TObject; Error: Word);
begin
   if Error <> 0 then exit;
   with Sender as TWSocket do
   begin
      Addr := DnsResult;
      Connect;
   end;
end;

//------------------------------------------------------------------------------
procedure TMain.RemoteSessionConnected(Sender: TObject; Error: Word);
begin
   if Error <> 0 then exit;
   with TWSocket(Sender).Owner as TClient do
   begin
      Log.TextColor := Color;
      Log.Header    := 'Connection Opened';
      Log.Add('');
      if Rcvd <> '' then
      begin
         Log.TextColor := Color;
         Log.Header    := 'From Local';
         Log.Add(Rcvd);
         RemoteSocket.SendStr(Rcvd);
         Rcvd := '';
      end;
   end;
end;

//------------------------------------------------------------------------------
procedure TMain.RemoteDataAvailable(Sender: TObject; Error: Word);
var FromRemote: string;
begin
   if Error <> 0 then exit;
   with TWSocket(Sender).Owner as TClient do
   begin
      FromRemote := RemoteSocket.ReceiveStr;
      if FromRemote <> '' then
      begin
         Log.TextColor := Color;
         Log.Header    := 'From Remote';
         Log.Add(FromRemote);
         if State = wsConnected then SendStr(FromRemote)
         else
         begin // should not occur
            Log.Header := 'Error: Local has closed';
            Log.Add('');
            RemoteSocket.CloseDelayed;
         end;
      end;
   end;
end;

//------------------------------------------------------------------------------
procedure TMain.ClientDataAvailable(Sender: TObject; Error: Word);
begin
   if Error <> 0 then exit;
   with Sender as TClient do
   begin
      Rcvd := Rcvd + ReceiveStr; // in case remote is still connecting
      if (RemoteSocket.State = wsConnected) and (Rcvd <> '') then
      begin
         Log.TextColor := Color;
         Log.Header    := 'From Local';
         Log.Add(Rcvd);
         RemoteSocket.SendStr(Rcvd);
         Rcvd := '';
      end;
   end;
end;

//------------------------------------------------------------------------------
procedure TMain.WSocketServerClientDisconnect(Sender: TObject; Client: TWSocketClient; Error: Word);
begin
   with Client as TClient do
   begin
      StatusBar.Panels[1].Text := IntToStr(TWSocketServer(Server).ClientCount - 1);
      if TWSocketServer(Server).Clientcount > 1 then StatusBar.Panels[3].Text := TWSocketServer(Server).Client[TWSocketServer(Server).ClientCount - 2].GetPeerAddr
      else StatusBar.Panels[3].Text := '';
   end;
end;

//------------------------------------------------------------------------------
procedure TMain.RemoteSessionClosed(Sender: TObject; Error: Word);
begin
   with TWSocket(Sender).Owner as TClient do ShutDown(1);
end;

//------------------------------------------------------------------------------
procedure TMain.BgException(Sender: TObject; E: Exception; var CanClose: Boolean);
begin
   CanClose := True;
end;

//------------------------------------------------------------------------------
procedure TMain.WSocketServerSessionClosed(Sender: TObject; Error: Word);
begin
   LocalPoort.Enabled := True;
   RemotePort.Enabled := True;
   RemoteAddr.Enabled := True;
   ListenBtn.Caption  := 'Listen';
   ListenBtn.Tag      :=  0;
end;

end.
